function [ F, alpha ] = MLEAVfit_twostate( trace, freq, X1, X2, T1 )
%[k,alpha] = MLEAVforce(trace,freq) loads a trace at freq, computes the Allan
%variance, fits the function from Lansdorp-2012 and returns the spring
%constant, k, and dissipation, alpha.  This
%version is modified to fit an average AV function for a twostate system.
%X1 and X2 are the extensions of the two states; T1 is the faction of the
%total time spent in state 1.

%Compute the Allan Variance of the time series:

[AV, tau] = AllanVar(trace, freq);

%Now we will need to do an unconstrained optimization to minimize the MLE
%cost function:

%Compute the trace-length parameters:

N = length(trace);
for k=1:length(tau)
    m(k) = 2^(k-1);
end;
eta = 1/2*(N./m-1);

%We use a least-squres fit to get the approximate parameters used in the
%maximum liklihood calculation:

ffun = fittype('T1*2*4.1*a/((f/X1)^2*x)*(1+2*a/((f/X1)*x)*exp(-(f/X1)*x/a)-a/(2*(f/X1)*x)*exp(-2*(f/X1)*x/a)-3*a/(2*(f/X1)*x))+(1-T1)*2*4.1*a/((f/X2)^2*x)*(1+2*a/((f/X2)*x)*exp(-(f/X2)*x/a)-a/(2*(f/X2)*x)*exp(-2*(f/X2)*x/a)-3*a/(2*(f/X2)*x))');
ffun_limited = fitoptions(ffun);
%Setting fit parameters:
set(ffun_limited, 'Lower', [1E-6,0], 'Upper',[1E-4,Inf], 'StartPoint',[2E-5,12],'MaxFunEvals',6000,'MaxIter',4000);
ffun = fittype('T1*2*4.1*a/((f/X1)^2*x)*(1+2*a/((f/X1)*x)*exp(-(f/X1)*x/a)-a/(2*(f/X1)*x)*exp(-2*(f/X1)*x/a)-3*a/(2*(f/X1)*x))+(1-T1)*2*4.1*a/((f/X2)^2*x)*(1+2*a/((f/X2)*x)*exp(-(f/X2)*x/a)-a/(2*(f/X2)*x)*exp(-2*(f/X2)*x/a)-3*a/(2*(f/X2)*x))', 'options', ffun_limited,'problem',{'X1','X2','T1'});
fitObject = fit(tau.',AV.', ffun,'problem',{X1,X2,T1});

coefficients = coeffvalues(fitObject);
alpha_guess = coefficients(1);
F_guess = coefficients(2);


%Define the anonymous function for the cost function in terms of two other
%functions defined at the end of this file: x = [a,k]

cost_fun = @(x) cost(x(1),x(2),AV,tau,eta,X1,X2,T1);

[fits,~,flag] = fminsearch(cost_fun,[alpha_guess,F_guess]);

%If the minimum search does not find a min., I return an error.  One could
%also handle this, on bad data where this is a common problem, by using the
%least-squares values obtained above

if flag ~= 1
    error('fminsearch minimization of cost function failed to converge');
end;

F = fits(2);
alpha = fits(1);
%disp(sprintf('Alpha: %g; k: %g',[alpha,k]));

%Make a plot the check the quality of the fit:
for n=1:length(tau)
   AV_fits(n) = AV_ideal(alpha,F,tau(n),X1,X2,T1);
end;


% figure;
% loglog(tau,AV,'.k');
% hold on;
% loglog(tau,AV_fits,'--r');
% title(sprintf('F: %2.2f; \alpha : %2.2f x 10^{-5}',[F,alpha*1e5]));


end

%Function for the AV from the Langevin equation:

function [AV] = AV_ideal(a,F,t,X1,X2,T1)
kT = 4.1;
AV = T1*2*kT*a/((F/X1)^2*t)*(1+2*a/((F/X1)*t)*exp(-(F/X1)*t/a)-a/(2*(F/X1)*t)*exp(-2*(F/X1)*t/a)-3*a/(2*(F/X1)*t))+(1-T1)*2*kT*a/((F/X2)^2*t)*(1+2*a/((F/X2)*t)*exp(-(F/X2)*t/a)-a/(2*(F/X2)*t)*exp(-2*(F/X2)*t/a)-3*a/(2*(F/X2)*t));
end

function [F] = cost(a,F,AV_measured,tau,eta,X1,X2,T1)
for n=1:length(tau)
    term(n) = eta(n)*(AV_measured(n)/AV_ideal(a,F,tau(n),X1,X2,T1)+log(AV_ideal(a,F,tau(n),X1,X2,T1)));  
end;
F = sum(term);
end

