function [ F, alpha ] = MLEPSDfit_twostate( trace, freq , X1,X2, T1)
%[k,alpha] = MLEPSDfit(trace,freq,X1,X2,T1) loads a trace at freq, computes the PSD
% fits the function from Lansdorp-2012 and returns the spring
%constant, k, and dissipation, alpha.   X1,X2 are the extensions of the 
%two states, T1 is the
%fraction of the total time spent in state 1

blocks = 30;

%Compute the PSD of the time series:

[PSD, f] = pwelch(trace, floor(length(trace)/blocks),[],[],freq,'twosided');

%Now we will need to do an unconstrained optimization to minimize the MLE
%cost function:

%For the PSD, the shape parameter is the number of blocks used:

eta = blocks;

%We use a least-squres fit to get the approximate parameters used in the
%maximum liklihood calculation:

ffun = fittype('T1*4.1/(2*3.14*a*((f/X1)^2/(4*9.87*a^2)+x^2))+T2*4.1/(2*3.14*a*((f/X2)^2/(4*9.87*a^2)+x^2))');
ffun_limited = fitoptions(ffun);
%Setting fit parameters:
set(ffun_limited, 'Lower', [1E-6,0], 'Upper',[1E-4,100], 'StartPoint',[2E-5,10],'MaxFunEvals',6000,'MaxIter',4000);
ffun = fittype('T1*4.1/(2*3.14*a*((f/X1)^2/(4*9.87*a^2)+x^2))+T2*4.1/(2*3.14*a*((f/X2)^2/(4*9.87*a^2)+x^2))', 'options', ffun_limited,'problem',{'T1','T2','X1','X2'});
fitObject = fit(f, PSD, ffun, 'problem',{T1,1-T1,X1,X2});


coefficients = coeffvalues(fitObject);
alpha_guess = coefficients(1);
F_guess = coefficients(2);


% figure;
% loglog(f,PSD,'-b');
% hold on;
% loglog(f,fitObject(f),'-r');
% title('approx. least squares fit');



%Define the anonymous function for the cost function in terms of two other
%functions defined at the end of this file: x = [a,k]

cost_fun = @(x) cost(x(1),x(2),PSD,f,eta, freq,X1,X2,T1);

[fits,~,flag] = fminsearch(cost_fun,[alpha_guess,F_guess]);

%If the minimum search does not find a min., I return an error.  One could
%also handle this, on bad data where this is a common problem, by using the
%least-squares values obtained above

if flag ~= 1
    error('fminsearch minimization of cost function failed to converge');
end;

F = abs(fits(2));
alpha = fits(1);

%disp(sprintf('Alpha: %g; k: %g',[alpha,k]));


%Make a plot the check the quality of the fit:

% for n=1:length(f)
%    PSD_fits(n) = PSD_ideal(alpha,F,f(n),freq,X1,X2,T1);
% end;

% figure;
% loglog(f,PSD,'.k');
% hold on;
% loglog(f,PSD_fits,'-r');
% title(sprintf('Force: %2.2f; alpha: %2.2f x 10^5',[F,alpha*1e5]));





end

%Function for the AV from the Langevin equation:

function [PSD] = PSD_ideal(a,F,f,fs,X1,X2,T1)
kT = 4.1;
% PSD = 2*kT*a/k^3*(k+(2*a*fs*sin(pi*f/fs)^2*sinh(k/a/fs))/(cos(2*pi*f/fs)-cosh(k/a/fs)));
% PSD = kT/(2*pi*a*(k^2/(4*pi^2*a^2)+f^2));
PSD = T1*2*kT*a/(F/X1)^3*((F/X1)+(2*a*fs*sin(pi*f/fs)^2*sinh((F/X1)/a/fs))/(cos(2*pi*f/fs)-cosh((F/X1)/a/fs)))+(1-T1)*2*kT*a/(F/X2)^3*((F/X2)+(2*a*fs*sin(pi*f/fs)^2*sinh((F/X2)/a/fs))/(cos(2*pi*f/fs)-cosh((F/X2)/a/fs)));
end

function [F] = cost(a,F,PSD_measured,f,eta, freq,X1,X2,T1)
for n=1:length(f)
    term(n) = eta*(PSD_measured(n)/PSD_ideal(a,F,f(n),freq,X1,X2,T1)+log(PSD_ideal(a,F,f(n),freq,X1,X2,T1)));  
end;
F = sum(term);
end

