%%% Sample simulation and fitting code to accompany:
%%%
%%% "Magnetic tweezers force calibration for molecules that 
%%%  exhibit conformational switching"
%%%
%%% David R. Jacobson & Omar A. Saleh, Rev. Sci. Instrum., 2016

%%% This script carries out a series of simulations with parameter values
%%% specified below.  The output of each simulation is then analyzed to
%%% obtain the Allan variance and power spectral density.  These spectra
%%% are subsequently fit to the various functions described in the paper to
%%% give force calibration estimates

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part 1: Parameter values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

F = 12;         %force applied to the molecule, in pN

X1 = 300;       %extension of the shorter state, in nm

X2 = 1300;      %extension of the longer state

rate = 1000;    %kinetic rate constant, in s^-1; this assumes that the 
                %system is at 50:50 folding/unfolding equilibrium, and is 
                %referred to as k_rate in the paper

alpha = 1.5e-5; %dissipation due to viscous drag, in pN s/nm; 
                %1.5e-5 is a typical value for an MT experiment 
                %using a 1-micron bead
                
sim_freq = 3e5; %frequency, in Hz, at which the simulation will be 
                %conducted; this should be several orders of magnitude 
                %faster than the downsampling frequency, below
                
freq = 3e3;     %simulated data will be downsampled by averaging to this 
                %frequency, akin to the camera frame rate   
                
sim_time = 10;  %length of simulation, in s

num_sims = 10;  %number of simulations to be conducted; the results of 
                %these will be combined to give estimates of uncertainty 
                %on the fit parameters
                
kT = 4.1;       %thermal energy, in pN nm (approx. 4.1 at room temp.)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part 2: Simulations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The output of these simulations are the downsampled x-traces, trace_ds{},
% a cell-array where each cell is the trace from one of the num_sims
% simulations.  The values of time corresponding to the points in these
% traces are given in time_ds

for sm = 1:num_sims     %loop over the number of times the simulation will 
                        %be replicated

    trace_length = sim_time*sim_freq;   %set number of data points 
                                        %in simulated trace

    dt = 1/sim_freq;    %length of time corresponding to each 
                        %data point

    k_avg = rate*dt;    %convert two-state switching rate, 
                        %specified as a number of transitions per 
                        %second, into a probability of switching 
                        %per dt

    %Since we assume 50:50 folding/unfolding equilibrium, both rate
    %constants are set to the average value.  This could be
    %modified to allow for simulation of a system away from the
    %coexistence condition:

    k_unfold = k_avg;
    k_fold = k_avg;

    %Parameters for the particular simulation are set up:
    Z_fold_tr = X1;         %extension of folded state
    Z_unfold_tr = X2;       %extension of unfolded state
    X_fold_tr = 0;          %avg. x-position of folded state
    X_unfold_tr = 0;        %avg. x-position of unfolded state
    k_spring_fold = F/Z_fold_tr;    %spring constant in z direction 
                                    %of folded state, from Eq. 1
    k_spring_unfold=F/Z_unfold_tr;  %spring constant of unfolded
    trace = zeros(trace_length,1);  %empty x-trace
    ztrace = zeros(trace_length,1); %empty z-trace
    state = 1;              %Starting state, 
                            %1 = folded, 2 = unfolded
    trace(1) = X_fold_tr;   %Seed x and z traces
    ztrace(1) = Z_fold_tr;

    %Running record of the nubmer of simulation points that the
    %system spends in each of the folded and unfolded states
    pts_fold = 0;
    pts_unfold = 0;

    %Pre-generate the random numbers that will be multiplied by the
    %appropriate pre-factor to give the Langevin force:
    F_L = randn(trace_length,1);

    %Iterative simulation, where n is the current frame of the
    %simulation
    for n=2:trace_length
        %Check to see if a stochastic state change is going to
        %occur.  If it does, change the value of 'state'.  Compute
        %the change in x-position, dx, based on the Langevin
        %equation.  Record the new value of the extension as z_new.
        %Note: fluctuations are only simulated in the x, not z,
        %direction
        if state == 1
            pts_fold = pts_fold +1;
            if rand <= k_unfold
                state = 2;
                dx = dt/alpha*(sqrt(2*alpha*kT/dt)*F_L(n)-k_spring_unfold*(trace(n-1)-X_unfold_tr));
                z_new = Z_unfold_tr;
            else
                dx = dt/alpha*(sqrt(2*alpha*kT/dt)*F_L(n)-k_spring_fold*(trace(n-1)-X_fold_tr));
                z_new = Z_fold_tr;
            end;
        else 
            pts_unfold = pts_unfold +1;
            if rand <= k_fold
                state = 1;
                dx = dt/alpha*(sqrt(2*alpha*kT/dt)*F_L(n)-k_spring_fold*(trace(n-1)-X_fold_tr));
                z_new = Z_fold_tr;
            else
                dx = dt/alpha*(sqrt(2*alpha*kT/dt)*F_L(n)-k_spring_unfold*(trace(n-1)-X_unfold_tr));
                z_new = Z_unfold_tr;
            end;
        end;

        %Add the results of this simulation increment to the
        %growing traces:

        trace(n) = trace(n-1) + dx;
        ztrace(n)= z_new;

    end;

    %The data are now downsampled by binning and averaging to
    %simulate the finite camera framerate:

    bin_size = floor(sim_freq/freq);
    num_bins = floor(length(trace)/bin_size);
    trace_ds{sm} = zeros(num_bins, 1);  %empty downsampled traces
    ztrace_ds{sm} = zeros(num_bins, 1); %
    time_ds = zeros(num_bins,1);    %array of time values 
                                    %corresponding to each point in 
                                    %the downsampled traces
    for k=1:num_bins
        trace_ds{sm}(k) = mean(trace(((k-1)*bin_size +1):k*bin_size));
        ztrace_ds{sm}(k) = mean(ztrace(((k-1)*bin_size +1):k*bin_size));
    end;


            
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part 3: Fitting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for sm = 1:num_sims     %again loops over the number of sim replicates
    
    %Average value of z-extension:
    z_avg = mean(ztrace_ds{sm});
    
    %Fraction of time that the system spends in state #1
    T1 = (length(trace_ds{sm})/2-sum(sign(trace_ds{sm})))/length(trace_ds{sm});

    %%% Compute the forces using the various methods, some of which call
    %%% external fitting functions in the package

    %Variance method (Eq. 2 of paper):

    F_var(sm) = z_avg*kT/var(trace_ds{sm});

    %Integrated PSD method:

    [psd, f] = pwelch(trace, [],[],[],freq,'twosided');
    F_ipsd(sm) = z_avg*kT/trapz(f, psd);

    %PSD, single-state method

    [k,~] = MLEPSDfit(trace_ds{sm},freq);
    F_psd1(sm) = z_avg*k;

    %AV, single-state method

    [k,~] = MLEAVfit(trace_ds{sm},freq);
    F_av1(sm) = z_avg*k;

    %PSD, double-state method

    [F_psd2(sm),~] = MLEPSDfit_twostate(trace_ds{sm},freq,X1,X2,T1);

    %AV, double-state method

    [F_av2(sm),~] = MLEAVfit_twostate(trace_ds{sm},freq,X1,X2,T1);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part 4: Caculation of force bias
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Values of the fit parameters are averaged across the simulations to find
%the percentage bias and the standard error of the mean of that value for
%each of the different force calibration methods.  These values are printed
%to the console.

disp('Variance:')
for xt = 1:length(X2)
    fprintf('dX: %d; Rate: %d; Force: %d \n', [X2-X1,rate,F]);
    fprintf('variance: %2.2f +/- %2.2f \n', [(F-mean(F_var))/F*100, std(F_var)/sqrt(num_sims)/F*100]); 
    fprintf('i-PSD:    %2.2f +/- %2.2f \n', [(F-mean(F_ipsd))/F*100, std(F_ipsd)/sqrt(num_sims)/F*100]); 
    fprintf('PSD (1) : %2.2f +/- %2.2f \n', [(F-mean(F_psd1))/F*100, std(F_psd1)/sqrt(num_sims)/F*100]);
    fprintf('PSD (2) : %2.2f +/- %2.2f \n', [(F-mean(F_psd2))/F*100, std(F_psd2)/sqrt(num_sims)/F*100]); 
    fprintf('AV  (1) : %2.2f +/- %2.2f \n', [(F-mean(F_av1))/F*100, std(F_av1)/sqrt(num_sims)/F*100]); 
    fprintf('AV  (2) : %2.2f +/- %2.2f \n\n', [(F-mean(F_av2))/F*100, std(F_av2)/sqrt(num_sims)/F*100]); 
end;



